#include "osHandles.h"              // Includes all OS Files
#include "System/cpu.h"             // Initialize CPU Hardware
#include "drivers/watchdog.h"
#include "drivers/uart0.h"       	// Initialize UART
#include "drivers/uart1.h"       	// Initialize UART
#include "utils/rprintf.h"			// Reduced printf.  STDIO uses lot of FLASH space & Stack space
#include "fat/diskio.h"
#include "general/userInterface.hpp"	// User interface functions to interact through UART
#include "MP3/mp3.hpp"
#include "utils/timer.hpp"
#include "tasks/blinkTask.hpp"
#include "tasks/motor.hpp"
#include "tasks/bluetooth.hpp"



/* INTERRUPT VECTORS:
 * 0:    OS Timer Tick
 * 1:    Not Used
 * 2:    UART0 Interrupt
 * 3:    Not Used
 * 4:    I2C0 Interrupt
 * 5:    Not Used
 * 6:    UART1 Interrupt
 * 5-16: Not Used
 */

int main (void)
{
	OSHANDLES SysHandles;        // Should contain all OS Handles

	cpuSetupHardware();          // Setup PLL, enable MAM etc.
	watchdogDelayUs(1000*1000);  // Some startup delay
	uart0Init(38400, 128);       // 128 is size of Rx/Tx FIFO

	// Use polling version of uart0 to do printf/rprintf before starting FreeRTOS
	rprintf_devopen(uart0PutCharPolling);
	cpuPrintMemoryInfo();

	// Open interrupt-driven version of UART0 Rx/Tx
	rprintf_devopen(uart0PutChar);
	watchdogDelayUs(1000*1000);

	/** Create any Queues and Mutexes **/

	SysHandles.lock.SPI = xSemaphoreCreateMutex();
	SysHandles.queue.command = xQueueCreate(1,8);
	SysHandles.queue.blink = xQueueCreate(1,8);

	// Use the WATERMARK command to determine optimal Stack size of each task (set to high, then slowly decrease)
	// Priorities should be set according to response required
	if (
		pdPASS != xTaskCreate( uartUI, (signed char*)"Uart UI", STACK_BYTES(1024*6), &SysHandles, PRIORITY_MEDIUM,  &SysHandles.task.userInterface )
		||
		pdPASS != xTaskCreate( blinkTask, (signed char*)"blinkTask", STACK_BYTES(1024), &SysHandles, PRIORITY_LOW,  &SysHandles.task.blinkTask )
		||
		pdPASS != xTaskCreate( motor, (signed char*)"motor", STACK_BYTES(1024), &SysHandles, PRIORITY_MEDIUM,  &SysHandles.task.motor )
		||
		pdPASS != xTaskCreate( bluetooth, (signed char*)"bluetooth", STACK_BYTES(1024), &SysHandles, PRIORITY_MEDIUM,  &SysHandles.task.bluetooth )
	){
		rprintf("ERROR:  OUT OF MEMORY: Check OS Stack Size or task stack size.\n");
	}

	// Start FreeRTOS to begin servicing tasks created above, vTaskStartScheduler() will not "return"
	rprintf("\n-- Starting FreeRTOS --\n");
	vTaskStartScheduler();

	// In case OS is exited:
	rprintf_devopen(uart0PutCharPolling);
	rprintf("ERROR: Unexpected OS Exit!\n");

	return 0;
}
